"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/no-empty-function */
const axios_1 = (0, tslib_1.__importDefault)(require("axios"));
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
const query_string_1 = (0, tslib_1.__importDefault)(require("query-string"));
class Auth {
    constructor(config, session) {
        const { baseURL, useCookies, refreshIntervalTime, clientStorage, clientStorageType, ssr, autoLogin, refreshTokenName, } = config;
        this.refreshTokenName = refreshTokenName;
        this.useCookies = useCookies;
        this.refreshIntervalTime = refreshIntervalTime;
        this.clientStorage = clientStorage;
        this.clientStorageType = clientStorageType;
        this.tokenChangedFunctions = [];
        this.authChangedFunctions = [];
        this.refreshSleepCheckInterval = 0;
        this.refreshIntervalSleepCheckLastSample = Date.now();
        this.sampleRate = 2000; // check every 2 seconds
        this.ssr = ssr;
        this.refreshTokenLock = false;
        this.baseURL = baseURL;
        this.loading = true;
        this.currentUser = null;
        this.currentSession = session;
        this.httpClient = axios_1.default.create({
            baseURL: `${this.baseURL}/auth`,
            timeout: 10000,
            withCredentials: this.useCookies,
        });
        // get refresh token from query param (from external OAuth provider callback)
        let refreshToken = null;
        if (!ssr) {
            try {
                const parsed = query_string_1.default.parse(window.location.search);
                refreshToken =
                    'refresh_token' in parsed ? parsed.refresh_token : null;
                if (refreshToken) {
                    const newURL = this._removeParam('refresh_token', window.location.href);
                    try {
                        window.history.pushState({}, document.title, newURL);
                    }
                    catch (_a) {
                        // noop
                        // window object not available
                    }
                }
            }
            catch (e) {
                // noop. `window` not available probably.
            }
        }
        // if empty string, then set it to null
        refreshToken = refreshToken ? refreshToken : null;
        if (autoLogin) {
            this._autoLogin(refreshToken);
        }
        else if (refreshToken) {
            this._setItem(this.refreshTokenName, refreshToken);
        }
    }
    user() {
        return this.currentUser;
    }
    register({ email, password, options = {}, }) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { userData, defaultRole, allowedRoles } = options;
            const registerOptions = defaultRole || allowedRoles
                ? {
                    default_role: defaultRole,
                    allowed_roles: allowedRoles,
                }
                : undefined;
            const res = yield this.httpClient.post('/register', {
                email,
                password,
                cookie: this.useCookies,
                user_data: userData,
                register_options: registerOptions,
            });
            if (res.data.jwt_token) {
                this._setSession(res.data);
                return { session: res.data, user: res.data.user };
            }
            else {
                // if AUTO_ACTIVATE_NEW_USERS is false
                return { session: null, user: res.data.user };
            }
        });
    }
    login({ email, password, provider, }) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (provider) {
                window.location.href = `${this.baseURL}/auth/providers/${provider}`;
                return { session: null, user: null };
            }
            let res;
            try {
                res = yield this.httpClient.post('/login', {
                    email,
                    password,
                    cookie: this.useCookies,
                });
            }
            catch (error) {
                this._clearSession();
                throw error;
            }
            if ('mfa' in res.data) {
                return { session: null, user: null, mfa: { ticket: res.data.ticket } };
            }
            yield this._setSession(res.data);
            return { session: res.data, user: res.data.user };
        });
    }
    logout(all = false) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.post('/logout', {
                    all,
                }, {
                    params: {
                        refresh_token: yield this._getItem(this.refreshTokenName),
                    },
                });
            }
            catch (error) {
                // throw error;
                // noop
            }
            yield this._clearSession();
            this._clearStorage();
            return { session: null, user: null };
        });
    }
    onTokenChanged(fn) {
        this.tokenChangedFunctions.push(fn);
        // get index;
        const tokenChangedFunctionIndex = this.authChangedFunctions.length - 1;
        const unsubscribe = () => {
            try {
                // replace onTokenChanged with empty function
                this.authChangedFunctions[tokenChangedFunctionIndex] = () => { };
            }
            catch (err) {
                console.warn('Unable to unsubscribe onTokenChanged function. Maybe you already did?');
            }
        };
        return unsubscribe;
    }
    onAuthStateChanged(fn) {
        this.authChangedFunctions.push(fn);
        // get index;
        const authStateChangedFunctionIndex = this.authChangedFunctions.length - 1;
        const unsubscribe = () => {
            try {
                // replace onAuthStateChanged with empty function
                this.authChangedFunctions[authStateChangedFunctionIndex] = () => { };
            }
            catch (err) {
                console.warn('Unable to unsubscribe onAuthStateChanged function. Maybe you already did?');
            }
        };
        return unsubscribe;
    }
    isAuthenticated() {
        if (this.loading)
            return null;
        return this.currentSession.getSession() !== null;
    }
    isAuthenticatedAsync() {
        const isAuthenticated = this.isAuthenticated();
        return new Promise((resolve) => {
            if (isAuthenticated !== null)
                resolve(isAuthenticated);
            else {
                const unsubscribe = this.onAuthStateChanged((isAuthenticated) => {
                    resolve(isAuthenticated);
                    unsubscribe();
                });
            }
        });
    }
    getJWTToken() {
        var _a;
        return ((_a = this.currentSession.getSession()) === null || _a === void 0 ? void 0 : _a.jwt_token) || null;
    }
    getUser() {
        var _a;
        return ((_a = this.currentSession.getSession()) === null || _a === void 0 ? void 0 : _a.user) || null;
    }
    getClaim(claim) {
        return this.currentSession.getClaim(claim);
    }
    refreshSession() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return yield this.refreshToken();
        });
    }
    activate(ticket) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.get(`/activate?ticket=${ticket}`);
        });
    }
    changeEmail(new_email) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/change-email', {
                new_email,
            }, {
                headers: this._generateHeaders(),
            });
        });
    }
    requestEmailChange(new_email) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/change-email/request', {
                new_email,
            }, {
                headers: this._generateHeaders(),
            });
        });
    }
    confirmEmailChange(ticket) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/change-email/change', {
                ticket,
            });
        });
    }
    changePassword({ oldPassword, newPassword, }) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/change-password', {
                old_password: oldPassword,
                new_password: newPassword,
            }, {
                headers: this._generateHeaders(),
            });
        });
    }
    requestPasswordChange(email) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/change-password/request', {
                email,
            });
        });
    }
    confirmPasswordChange(newPassword, ticket) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/change-password/change', {
                new_password: newPassword,
                ticket,
            });
        });
    }
    MFAGenerate() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const res = yield this.httpClient.post('/mfa/generate', {}, {
                headers: this._generateHeaders(),
            });
            return res.data;
        });
    }
    MFAEnable(code) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/mfa/enable', {
                code,
            }, {
                headers: this._generateHeaders(),
            });
        });
    }
    MFADisable(code) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.httpClient.post('/mfa/disable', {
                code,
            }, {
                headers: this._generateHeaders(),
            });
        });
    }
    MFATotp(code, ticket) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const res = yield this.httpClient.post('/mfa/totp', {
                code,
                ticket,
                cookie: this.useCookies,
            });
            this._setSession(res.data);
            return { session: res.data, user: res.data.user };
        });
    }
    _removeParam(key, sourceURL) {
        let rtn = sourceURL.split('?')[0], param, params_arr = [];
        const queryString = sourceURL.indexOf('?') !== -1 ? sourceURL.split('?')[1] : '';
        if (queryString !== '') {
            params_arr = queryString.split('&');
            for (let i = params_arr.length - 1; i >= 0; i -= 1) {
                param = params_arr[i].split('=')[0];
                if (param === key) {
                    params_arr.splice(i, 1);
                }
            }
            if (params_arr.length > 0) {
                rtn = rtn + '?' + params_arr.join('&');
            }
        }
        return rtn;
    }
    _setItem(key, value) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (typeof value !== 'string') {
                console.error(`value is not of type "string"`);
                return;
            }
            switch (this.clientStorageType) {
                case 'web':
                    if (typeof this.clientStorage.setItem !== 'function') {
                        console.error(`this.clientStorage.setItem is not a function`);
                        break;
                    }
                    this.clientStorage.setItem(key, value);
                    break;
                case 'custom':
                case 'react-native':
                    if (typeof this.clientStorage.setItem !== 'function') {
                        console.error(`this.clientStorage.setItem is not a function`);
                        break;
                    }
                    yield this.clientStorage.setItem(key, value);
                    break;
                case 'capacitor':
                    if (typeof this.clientStorage.set !== 'function') {
                        console.error(`this.clientStorage.set is not a function`);
                        break;
                    }
                    yield this.clientStorage.set({ key, value });
                    break;
                case 'expo-secure-storage':
                    if (typeof this.clientStorage.setItemAsync !== 'function') {
                        console.error(`this.clientStorage.setItemAsync is not a function`);
                        break;
                    }
                    this.clientStorage.setItemAsync(key, value);
                    break;
                default:
                    break;
            }
        });
    }
    _getItem(key) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            switch (this.clientStorageType) {
                case 'web':
                    if (typeof this.clientStorage.getItem !== 'function') {
                        console.error(`this.clientStorage.getItem is not a function`);
                        break;
                    }
                    return this.clientStorage.getItem(key);
                case 'custom':
                case 'react-native':
                    if (typeof this.clientStorage.getItem !== 'function') {
                        console.error(`this.clientStorage.getItem is not a function`);
                        break;
                    }
                    return yield this.clientStorage.getItem(key);
                case 'capacitor':
                    if (typeof this.clientStorage.get !== 'function') {
                        console.error(`this.clientStorage.get is not a function`);
                        break;
                    }
                    return (yield this.clientStorage.get({ key })).value;
                case 'expo-secure-storage':
                    if (typeof this.clientStorage.getItemAsync !== 'function') {
                        console.error(`this.clientStorage.getItemAsync is not a function`);
                        break;
                    }
                    return this.clientStorage.getItemAsync(key);
                default:
                    break;
            }
            return;
        });
    }
    _removeItem(key) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            switch (this.clientStorageType) {
                case 'web':
                    if (typeof this.clientStorage.removeItem !== 'function') {
                        console.error(`this.clientStorage.removeItem is not a function`);
                        break;
                    }
                    return this.clientStorage.removeItem(key);
                case 'custom':
                case 'react-native':
                    if (typeof this.clientStorage.removeItem !== 'function') {
                        console.error(`this.clientStorage.removeItem is not a function`);
                        break;
                    }
                    return yield this.clientStorage.removeItem(key);
                case 'capacitor':
                    if (typeof this.clientStorage.remove !== 'function') {
                        console.error(`this.clientStorage.remove is not a function`);
                        break;
                    }
                    yield this.clientStorage.remove({ key });
                    break;
                case 'expo-secure-storage':
                    if (typeof this.clientStorage.deleteItemAsync !== 'function') {
                        console.error(`this.clientStorage.deleteItemAsync is not a function`);
                        break;
                    }
                    this.clientStorage.deleteItemAsync(key);
                    break;
                default:
                    break;
            }
        });
    }
    _generateHeaders() {
        var _a;
        if (this.useCookies)
            return;
        return {
            Authorization: `Bearer ${(_a = this.currentSession.getSession()) === null || _a === void 0 ? void 0 : _a.jwt_token}`,
        };
    }
    _autoLogin(refreshToken) {
        if (this.ssr) {
            return;
        }
        this.refreshToken(refreshToken);
    }
    refreshToken(initRefreshToken) {
        var _a;
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const refreshToken = initRefreshToken || (yield this._getItem(this.refreshTokenName));
            if (!refreshToken) {
                // place at end of call-stack to let frontend get `null` first (to match SSR)
                setTimeout(() => {
                    this._clearSession();
                }, 0);
                return;
            }
            let res;
            try {
                // set lock to avoid two refresh token request being sent at the same time with the same token.
                // If so, the last request will fail because the first request used the refresh token
                if (this.refreshTokenLock) {
                    return;
                }
                this.refreshTokenLock = true;
                // make refresh token request
                res = yield this.httpClient.get('/token/refresh', {
                    params: {
                        refresh_token: refreshToken,
                    },
                });
            }
            catch (error) {
                if (((_a = error.response) === null || _a === void 0 ? void 0 : _a.status) === 401) {
                    yield this.logout();
                    return;
                }
                else {
                    return; // silent fail
                }
            }
            finally {
                // release lock
                this.refreshTokenLock = false;
            }
            yield this._setSession(res.data);
            this.tokenChanged();
        });
    }
    tokenChanged() {
        for (const tokenChangedFunction of this.tokenChangedFunctions) {
            tokenChangedFunction();
        }
    }
    authStateChanged(state) {
        for (const authChangedFunction of this.authChangedFunctions) {
            authChangedFunction(state);
        }
    }
    _clearStorage() {
        localStorage.removeItem(this.refreshTokenName || 'refreshToken');
    }
    _clearSession() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            // early exit
            if (this.isAuthenticated() === false) {
                return;
            }
            clearInterval(this.refreshInterval);
            clearInterval(this.refreshSleepCheckInterval);
            this.currentSession.clearSession();
            this._removeItem(this.refreshTokenName);
            this.loading = false;
            this.authStateChanged(false);
        });
    }
    _setSession(session) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const previouslyAuthenticated = this.isAuthenticated();
            this.currentSession.setSession(session);
            this.currentUser = session.user;
            if (!this.useCookies && session.refresh_token) {
                yield this._setItem(this.refreshTokenName, session.refresh_token);
            }
            if (!previouslyAuthenticated) {
                // start refresh token interval after logging in
                const JWTExpiresIn = session.jwt_expires_in;
                const refreshIntervalTime = this.refreshIntervalTime
                    ? this.refreshIntervalTime
                    : Math.max(30 * 1000, JWTExpiresIn - 45000); //45 sec before expires
                this.refreshInterval = setInterval(this.refreshToken.bind(this), refreshIntervalTime);
                // refresh token after computer has been sleeping
                // https://stackoverflow.com/questions/14112708/start-calling-js-function-when-pc-wakeup-from-sleep-mode
                this.refreshIntervalSleepCheckLastSample = Date.now();
                this.refreshSleepCheckInterval = setInterval(() => {
                    if (Date.now() - this.refreshIntervalSleepCheckLastSample >=
                        this.sampleRate * 2) {
                        this.refreshToken();
                    }
                    this.refreshIntervalSleepCheckLastSample = Date.now();
                }, this.sampleRate);
                this.authStateChanged(true);
            }
            this.loading = false;
        });
    }
}
exports.default = Auth;
//# sourceMappingURL=Auth.js.map